/*
 * DO NOT EDIT.  THIS FILE IS GENERATED FROM ../../../dist/idl/nsIHTMLAbsPosEditor.idl
 */

#ifndef __gen_nsIHTMLAbsPosEditor_h__
#define __gen_nsIHTMLAbsPosEditor_h__


#ifndef __gen_nsISupports_h__
#include "nsISupports.h"
#endif

#ifndef __gen_domstubs_h__
#include "domstubs.h"
#endif

/* For IDL files that don't want to include root IDL files. */
#ifndef NS_NO_VTABLE
#define NS_NO_VTABLE
#endif

/* starting interface:    nsIHTMLAbsPosEditor */
#define NS_IHTMLABSPOSEDITOR_IID_STR "91375f52-20e6-4757-9835-eb04fabe5498"

#define NS_IHTMLABSPOSEDITOR_IID \
  {0x91375f52, 0x20e6, 0x4757, \
    { 0x98, 0x35, 0xeb, 0x04, 0xfa, 0xbe, 0x54, 0x98 }}

class NS_NO_VTABLE nsIHTMLAbsPosEditor : public nsISupports {
 public:

  NS_DECLARE_STATIC_IID_ACCESSOR(NS_IHTMLABSPOSEDITOR_IID)

  /* readonly attribute boolean selectionContainerAbsolutelyPositioned; */
  NS_IMETHOD GetSelectionContainerAbsolutelyPositioned(bool *aSelectionContainerAbsolutelyPositioned) = 0;

  /* readonly attribute nsIDOMElement positionedElement; */
  NS_IMETHOD GetPositionedElement(nsIDOMElement * *aPositionedElement) = 0;

  /* attribute boolean absolutePositioningEnabled; */
  NS_IMETHOD GetAbsolutePositioningEnabled(bool *aAbsolutePositioningEnabled) = 0;
  NS_IMETHOD SetAbsolutePositioningEnabled(bool aAbsolutePositioningEnabled) = 0;

  /* attribute boolean snapToGridEnabled; */
  NS_IMETHOD GetSnapToGridEnabled(bool *aSnapToGridEnabled) = 0;
  NS_IMETHOD SetSnapToGridEnabled(bool aSnapToGridEnabled) = 0;

  /* attribute unsigned long gridSize; */
  NS_IMETHOD GetGridSize(uint32_t *aGridSize) = 0;
  NS_IMETHOD SetGridSize(uint32_t aGridSize) = 0;

  /* readonly attribute nsIDOMElement absolutelyPositionedSelectionContainer; */
  NS_IMETHOD GetAbsolutelyPositionedSelectionContainer(nsIDOMElement * *aAbsolutelyPositionedSelectionContainer) = 0;

  /* void absolutePositionSelection (in boolean aEnabled); */
  NS_IMETHOD AbsolutePositionSelection(bool aEnabled) = 0;

  /* void relativeChangeZIndex (in long aChange); */
  NS_IMETHOD RelativeChangeZIndex(int32_t aChange) = 0;

  /* void absolutelyPositionElement (in nsIDOMElement aElement, in boolean aEnabled); */
  NS_IMETHOD AbsolutelyPositionElement(nsIDOMElement *aElement, bool aEnabled) = 0;

  /* void setElementPosition (in nsIDOMElement aElement, in long aX, in long aY); */
  NS_IMETHOD SetElementPosition(nsIDOMElement *aElement, int32_t aX, int32_t aY) = 0;

  /* long getElementZIndex (in nsIDOMElement aElement); */
  NS_IMETHOD GetElementZIndex(nsIDOMElement *aElement, int32_t *_retval) = 0;

  /* void setElementZIndex (in nsIDOMElement aElement, in long aZorder); */
  NS_IMETHOD SetElementZIndex(nsIDOMElement *aElement, int32_t aZorder) = 0;

  /* long relativeChangeElementZIndex (in nsIDOMElement aElement, in long aChange); */
  NS_IMETHOD RelativeChangeElementZIndex(nsIDOMElement *aElement, int32_t aChange, int32_t *_retval) = 0;

  /* void showGrabberOnElement (in nsIDOMElement aElement); */
  NS_IMETHOD ShowGrabberOnElement(nsIDOMElement *aElement) = 0;

  /* void hideGrabber (); */
  NS_IMETHOD HideGrabber(void) = 0;

  /* void refreshGrabber (); */
  NS_IMETHOD RefreshGrabber(void) = 0;

};

  NS_DEFINE_STATIC_IID_ACCESSOR(nsIHTMLAbsPosEditor, NS_IHTMLABSPOSEDITOR_IID)

/* Use this macro when declaring classes that implement this interface. */
#define NS_DECL_NSIHTMLABSPOSEDITOR \
  NS_IMETHOD GetSelectionContainerAbsolutelyPositioned(bool *aSelectionContainerAbsolutelyPositioned) override; \
  NS_IMETHOD GetPositionedElement(nsIDOMElement * *aPositionedElement) override; \
  NS_IMETHOD GetAbsolutePositioningEnabled(bool *aAbsolutePositioningEnabled) override; \
  NS_IMETHOD SetAbsolutePositioningEnabled(bool aAbsolutePositioningEnabled) override; \
  NS_IMETHOD GetSnapToGridEnabled(bool *aSnapToGridEnabled) override; \
  NS_IMETHOD SetSnapToGridEnabled(bool aSnapToGridEnabled) override; \
  NS_IMETHOD GetGridSize(uint32_t *aGridSize) override; \
  NS_IMETHOD SetGridSize(uint32_t aGridSize) override; \
  NS_IMETHOD GetAbsolutelyPositionedSelectionContainer(nsIDOMElement * *aAbsolutelyPositionedSelectionContainer) override; \
  NS_IMETHOD AbsolutePositionSelection(bool aEnabled) override; \
  NS_IMETHOD RelativeChangeZIndex(int32_t aChange) override; \
  NS_IMETHOD AbsolutelyPositionElement(nsIDOMElement *aElement, bool aEnabled) override; \
  NS_IMETHOD SetElementPosition(nsIDOMElement *aElement, int32_t aX, int32_t aY) override; \
  NS_IMETHOD GetElementZIndex(nsIDOMElement *aElement, int32_t *_retval) override; \
  NS_IMETHOD SetElementZIndex(nsIDOMElement *aElement, int32_t aZorder) override; \
  NS_IMETHOD RelativeChangeElementZIndex(nsIDOMElement *aElement, int32_t aChange, int32_t *_retval) override; \
  NS_IMETHOD ShowGrabberOnElement(nsIDOMElement *aElement) override; \
  NS_IMETHOD HideGrabber(void) override; \
  NS_IMETHOD RefreshGrabber(void) override; 

/* Use this macro when declaring the members of this interface when the
   class doesn't implement the interface. This is useful for forwarding. */
#define NS_DECL_NON_VIRTUAL_NSIHTMLABSPOSEDITOR \
  NS_METHOD GetSelectionContainerAbsolutelyPositioned(bool *aSelectionContainerAbsolutelyPositioned); \
  NS_METHOD GetPositionedElement(nsIDOMElement * *aPositionedElement); \
  NS_METHOD GetAbsolutePositioningEnabled(bool *aAbsolutePositioningEnabled); \
  NS_METHOD SetAbsolutePositioningEnabled(bool aAbsolutePositioningEnabled); \
  NS_METHOD GetSnapToGridEnabled(bool *aSnapToGridEnabled); \
  NS_METHOD SetSnapToGridEnabled(bool aSnapToGridEnabled); \
  NS_METHOD GetGridSize(uint32_t *aGridSize); \
  NS_METHOD SetGridSize(uint32_t aGridSize); \
  NS_METHOD GetAbsolutelyPositionedSelectionContainer(nsIDOMElement * *aAbsolutelyPositionedSelectionContainer); \
  NS_METHOD AbsolutePositionSelection(bool aEnabled); \
  NS_METHOD RelativeChangeZIndex(int32_t aChange); \
  NS_METHOD AbsolutelyPositionElement(nsIDOMElement *aElement, bool aEnabled); \
  NS_METHOD SetElementPosition(nsIDOMElement *aElement, int32_t aX, int32_t aY); \
  NS_METHOD GetElementZIndex(nsIDOMElement *aElement, int32_t *_retval); \
  NS_METHOD SetElementZIndex(nsIDOMElement *aElement, int32_t aZorder); \
  NS_METHOD RelativeChangeElementZIndex(nsIDOMElement *aElement, int32_t aChange, int32_t *_retval); \
  NS_METHOD ShowGrabberOnElement(nsIDOMElement *aElement); \
  NS_METHOD HideGrabber(void); \
  NS_METHOD RefreshGrabber(void); 

/* Use this macro to declare functions that forward the behavior of this interface to another object. */
#define NS_FORWARD_NSIHTMLABSPOSEDITOR(_to) \
  NS_IMETHOD GetSelectionContainerAbsolutelyPositioned(bool *aSelectionContainerAbsolutelyPositioned) override { return _to GetSelectionContainerAbsolutelyPositioned(aSelectionContainerAbsolutelyPositioned); } \
  NS_IMETHOD GetPositionedElement(nsIDOMElement * *aPositionedElement) override { return _to GetPositionedElement(aPositionedElement); } \
  NS_IMETHOD GetAbsolutePositioningEnabled(bool *aAbsolutePositioningEnabled) override { return _to GetAbsolutePositioningEnabled(aAbsolutePositioningEnabled); } \
  NS_IMETHOD SetAbsolutePositioningEnabled(bool aAbsolutePositioningEnabled) override { return _to SetAbsolutePositioningEnabled(aAbsolutePositioningEnabled); } \
  NS_IMETHOD GetSnapToGridEnabled(bool *aSnapToGridEnabled) override { return _to GetSnapToGridEnabled(aSnapToGridEnabled); } \
  NS_IMETHOD SetSnapToGridEnabled(bool aSnapToGridEnabled) override { return _to SetSnapToGridEnabled(aSnapToGridEnabled); } \
  NS_IMETHOD GetGridSize(uint32_t *aGridSize) override { return _to GetGridSize(aGridSize); } \
  NS_IMETHOD SetGridSize(uint32_t aGridSize) override { return _to SetGridSize(aGridSize); } \
  NS_IMETHOD GetAbsolutelyPositionedSelectionContainer(nsIDOMElement * *aAbsolutelyPositionedSelectionContainer) override { return _to GetAbsolutelyPositionedSelectionContainer(aAbsolutelyPositionedSelectionContainer); } \
  NS_IMETHOD AbsolutePositionSelection(bool aEnabled) override { return _to AbsolutePositionSelection(aEnabled); } \
  NS_IMETHOD RelativeChangeZIndex(int32_t aChange) override { return _to RelativeChangeZIndex(aChange); } \
  NS_IMETHOD AbsolutelyPositionElement(nsIDOMElement *aElement, bool aEnabled) override { return _to AbsolutelyPositionElement(aElement, aEnabled); } \
  NS_IMETHOD SetElementPosition(nsIDOMElement *aElement, int32_t aX, int32_t aY) override { return _to SetElementPosition(aElement, aX, aY); } \
  NS_IMETHOD GetElementZIndex(nsIDOMElement *aElement, int32_t *_retval) override { return _to GetElementZIndex(aElement, _retval); } \
  NS_IMETHOD SetElementZIndex(nsIDOMElement *aElement, int32_t aZorder) override { return _to SetElementZIndex(aElement, aZorder); } \
  NS_IMETHOD RelativeChangeElementZIndex(nsIDOMElement *aElement, int32_t aChange, int32_t *_retval) override { return _to RelativeChangeElementZIndex(aElement, aChange, _retval); } \
  NS_IMETHOD ShowGrabberOnElement(nsIDOMElement *aElement) override { return _to ShowGrabberOnElement(aElement); } \
  NS_IMETHOD HideGrabber(void) override { return _to HideGrabber(); } \
  NS_IMETHOD RefreshGrabber(void) override { return _to RefreshGrabber(); } 

/* Use this macro to declare functions that forward the behavior of this interface to another object in a safe way. */
#define NS_FORWARD_SAFE_NSIHTMLABSPOSEDITOR(_to) \
  NS_IMETHOD GetSelectionContainerAbsolutelyPositioned(bool *aSelectionContainerAbsolutelyPositioned) override { return !_to ? NS_ERROR_NULL_POINTER : _to->GetSelectionContainerAbsolutelyPositioned(aSelectionContainerAbsolutelyPositioned); } \
  NS_IMETHOD GetPositionedElement(nsIDOMElement * *aPositionedElement) override { return !_to ? NS_ERROR_NULL_POINTER : _to->GetPositionedElement(aPositionedElement); } \
  NS_IMETHOD GetAbsolutePositioningEnabled(bool *aAbsolutePositioningEnabled) override { return !_to ? NS_ERROR_NULL_POINTER : _to->GetAbsolutePositioningEnabled(aAbsolutePositioningEnabled); } \
  NS_IMETHOD SetAbsolutePositioningEnabled(bool aAbsolutePositioningEnabled) override { return !_to ? NS_ERROR_NULL_POINTER : _to->SetAbsolutePositioningEnabled(aAbsolutePositioningEnabled); } \
  NS_IMETHOD GetSnapToGridEnabled(bool *aSnapToGridEnabled) override { return !_to ? NS_ERROR_NULL_POINTER : _to->GetSnapToGridEnabled(aSnapToGridEnabled); } \
  NS_IMETHOD SetSnapToGridEnabled(bool aSnapToGridEnabled) override { return !_to ? NS_ERROR_NULL_POINTER : _to->SetSnapToGridEnabled(aSnapToGridEnabled); } \
  NS_IMETHOD GetGridSize(uint32_t *aGridSize) override { return !_to ? NS_ERROR_NULL_POINTER : _to->GetGridSize(aGridSize); } \
  NS_IMETHOD SetGridSize(uint32_t aGridSize) override { return !_to ? NS_ERROR_NULL_POINTER : _to->SetGridSize(aGridSize); } \
  NS_IMETHOD GetAbsolutelyPositionedSelectionContainer(nsIDOMElement * *aAbsolutelyPositionedSelectionContainer) override { return !_to ? NS_ERROR_NULL_POINTER : _to->GetAbsolutelyPositionedSelectionContainer(aAbsolutelyPositionedSelectionContainer); } \
  NS_IMETHOD AbsolutePositionSelection(bool aEnabled) override { return !_to ? NS_ERROR_NULL_POINTER : _to->AbsolutePositionSelection(aEnabled); } \
  NS_IMETHOD RelativeChangeZIndex(int32_t aChange) override { return !_to ? NS_ERROR_NULL_POINTER : _to->RelativeChangeZIndex(aChange); } \
  NS_IMETHOD AbsolutelyPositionElement(nsIDOMElement *aElement, bool aEnabled) override { return !_to ? NS_ERROR_NULL_POINTER : _to->AbsolutelyPositionElement(aElement, aEnabled); } \
  NS_IMETHOD SetElementPosition(nsIDOMElement *aElement, int32_t aX, int32_t aY) override { return !_to ? NS_ERROR_NULL_POINTER : _to->SetElementPosition(aElement, aX, aY); } \
  NS_IMETHOD GetElementZIndex(nsIDOMElement *aElement, int32_t *_retval) override { return !_to ? NS_ERROR_NULL_POINTER : _to->GetElementZIndex(aElement, _retval); } \
  NS_IMETHOD SetElementZIndex(nsIDOMElement *aElement, int32_t aZorder) override { return !_to ? NS_ERROR_NULL_POINTER : _to->SetElementZIndex(aElement, aZorder); } \
  NS_IMETHOD RelativeChangeElementZIndex(nsIDOMElement *aElement, int32_t aChange, int32_t *_retval) override { return !_to ? NS_ERROR_NULL_POINTER : _to->RelativeChangeElementZIndex(aElement, aChange, _retval); } \
  NS_IMETHOD ShowGrabberOnElement(nsIDOMElement *aElement) override { return !_to ? NS_ERROR_NULL_POINTER : _to->ShowGrabberOnElement(aElement); } \
  NS_IMETHOD HideGrabber(void) override { return !_to ? NS_ERROR_NULL_POINTER : _to->HideGrabber(); } \
  NS_IMETHOD RefreshGrabber(void) override { return !_to ? NS_ERROR_NULL_POINTER : _to->RefreshGrabber(); } 

#if 0
/* Use the code below as a template for the implementation class for this interface. */

/* Header file */
class nsHTMLAbsPosEditor : public nsIHTMLAbsPosEditor
{
public:
  NS_DECL_ISUPPORTS
  NS_DECL_NSIHTMLABSPOSEDITOR

  nsHTMLAbsPosEditor();

private:
  ~nsHTMLAbsPosEditor();

protected:
  /* additional members */
};

/* Implementation file */
NS_IMPL_ISUPPORTS(nsHTMLAbsPosEditor, nsIHTMLAbsPosEditor)

nsHTMLAbsPosEditor::nsHTMLAbsPosEditor()
{
  /* member initializers and constructor code */
}

nsHTMLAbsPosEditor::~nsHTMLAbsPosEditor()
{
  /* destructor code */
}

/* readonly attribute boolean selectionContainerAbsolutelyPositioned; */
NS_IMETHODIMP nsHTMLAbsPosEditor::GetSelectionContainerAbsolutelyPositioned(bool *aSelectionContainerAbsolutelyPositioned)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* readonly attribute nsIDOMElement positionedElement; */
NS_IMETHODIMP nsHTMLAbsPosEditor::GetPositionedElement(nsIDOMElement * *aPositionedElement)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* attribute boolean absolutePositioningEnabled; */
NS_IMETHODIMP nsHTMLAbsPosEditor::GetAbsolutePositioningEnabled(bool *aAbsolutePositioningEnabled)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}
NS_IMETHODIMP nsHTMLAbsPosEditor::SetAbsolutePositioningEnabled(bool aAbsolutePositioningEnabled)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* attribute boolean snapToGridEnabled; */
NS_IMETHODIMP nsHTMLAbsPosEditor::GetSnapToGridEnabled(bool *aSnapToGridEnabled)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}
NS_IMETHODIMP nsHTMLAbsPosEditor::SetSnapToGridEnabled(bool aSnapToGridEnabled)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* attribute unsigned long gridSize; */
NS_IMETHODIMP nsHTMLAbsPosEditor::GetGridSize(uint32_t *aGridSize)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}
NS_IMETHODIMP nsHTMLAbsPosEditor::SetGridSize(uint32_t aGridSize)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* readonly attribute nsIDOMElement absolutelyPositionedSelectionContainer; */
NS_IMETHODIMP nsHTMLAbsPosEditor::GetAbsolutelyPositionedSelectionContainer(nsIDOMElement * *aAbsolutelyPositionedSelectionContainer)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* void absolutePositionSelection (in boolean aEnabled); */
NS_IMETHODIMP nsHTMLAbsPosEditor::AbsolutePositionSelection(bool aEnabled)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* void relativeChangeZIndex (in long aChange); */
NS_IMETHODIMP nsHTMLAbsPosEditor::RelativeChangeZIndex(int32_t aChange)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* void absolutelyPositionElement (in nsIDOMElement aElement, in boolean aEnabled); */
NS_IMETHODIMP nsHTMLAbsPosEditor::AbsolutelyPositionElement(nsIDOMElement *aElement, bool aEnabled)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* void setElementPosition (in nsIDOMElement aElement, in long aX, in long aY); */
NS_IMETHODIMP nsHTMLAbsPosEditor::SetElementPosition(nsIDOMElement *aElement, int32_t aX, int32_t aY)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* long getElementZIndex (in nsIDOMElement aElement); */
NS_IMETHODIMP nsHTMLAbsPosEditor::GetElementZIndex(nsIDOMElement *aElement, int32_t *_retval)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* void setElementZIndex (in nsIDOMElement aElement, in long aZorder); */
NS_IMETHODIMP nsHTMLAbsPosEditor::SetElementZIndex(nsIDOMElement *aElement, int32_t aZorder)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* long relativeChangeElementZIndex (in nsIDOMElement aElement, in long aChange); */
NS_IMETHODIMP nsHTMLAbsPosEditor::RelativeChangeElementZIndex(nsIDOMElement *aElement, int32_t aChange, int32_t *_retval)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* void showGrabberOnElement (in nsIDOMElement aElement); */
NS_IMETHODIMP nsHTMLAbsPosEditor::ShowGrabberOnElement(nsIDOMElement *aElement)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* void hideGrabber (); */
NS_IMETHODIMP nsHTMLAbsPosEditor::HideGrabber()
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* void refreshGrabber (); */
NS_IMETHODIMP nsHTMLAbsPosEditor::RefreshGrabber()
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* End of implementation class template. */
#endif


#endif /* __gen_nsIHTMLAbsPosEditor_h__ */
