/*
 * DO NOT EDIT.  THIS FILE IS GENERATED FROM ../../../dist/idl/nsIPushNotificationService.idl
 */

#ifndef __gen_nsIPushNotificationService_h__
#define __gen_nsIPushNotificationService_h__


#ifndef __gen_nsISupports_h__
#include "nsISupports.h"
#endif

#include "js/Value.h"

/* For IDL files that don't want to include root IDL files. */
#ifndef NS_NO_VTABLE
#define NS_NO_VTABLE
#endif

/* starting interface:    nsIPushNotificationService */
#define NS_IPUSHNOTIFICATIONSERVICE_IID_STR "74586476-d73f-4867-bece-87c1dea35750"

#define NS_IPUSHNOTIFICATIONSERVICE_IID \
  {0x74586476, 0xd73f, 0x4867, \
    { 0xbe, 0xce, 0x87, 0xc1, 0xde, 0xa3, 0x57, 0x50 }}

class NS_NO_VTABLE nsIPushNotificationService : public nsISupports {
 public:

  NS_DECLARE_STATIC_IID_ACCESSOR(NS_IPUSHNOTIFICATIONSERVICE_IID)

  /* jsval register (in string scope, in jsval originAttributes); */
  NS_IMETHOD Register(const char * scope, JS::HandleValue originAttributes, JS::MutableHandleValue _retval) = 0;

  /* jsval unregister (in string scope, in jsval originAttributes); */
  NS_IMETHOD Unregister(const char * scope, JS::HandleValue originAttributes, JS::MutableHandleValue _retval) = 0;

  /* jsval registration (in string scope, in jsval originAttributes); */
  NS_IMETHOD Registration(const char * scope, JS::HandleValue originAttributes, JS::MutableHandleValue _retval) = 0;

  /* jsval clearAll (); */
  NS_IMETHOD ClearAll(JS::MutableHandleValue _retval) = 0;

  /* jsval clearForDomain (in string domain); */
  NS_IMETHOD ClearForDomain(const char * domain, JS::MutableHandleValue _retval) = 0;

};

  NS_DEFINE_STATIC_IID_ACCESSOR(nsIPushNotificationService, NS_IPUSHNOTIFICATIONSERVICE_IID)

/* Use this macro when declaring classes that implement this interface. */
#define NS_DECL_NSIPUSHNOTIFICATIONSERVICE \
  NS_IMETHOD Register(const char * scope, JS::HandleValue originAttributes, JS::MutableHandleValue _retval) override; \
  NS_IMETHOD Unregister(const char * scope, JS::HandleValue originAttributes, JS::MutableHandleValue _retval) override; \
  NS_IMETHOD Registration(const char * scope, JS::HandleValue originAttributes, JS::MutableHandleValue _retval) override; \
  NS_IMETHOD ClearAll(JS::MutableHandleValue _retval) override; \
  NS_IMETHOD ClearForDomain(const char * domain, JS::MutableHandleValue _retval) override; 

/* Use this macro when declaring the members of this interface when the
   class doesn't implement the interface. This is useful for forwarding. */
#define NS_DECL_NON_VIRTUAL_NSIPUSHNOTIFICATIONSERVICE \
  NS_METHOD Register(const char * scope, JS::HandleValue originAttributes, JS::MutableHandleValue _retval); \
  NS_METHOD Unregister(const char * scope, JS::HandleValue originAttributes, JS::MutableHandleValue _retval); \
  NS_METHOD Registration(const char * scope, JS::HandleValue originAttributes, JS::MutableHandleValue _retval); \
  NS_METHOD ClearAll(JS::MutableHandleValue _retval); \
  NS_METHOD ClearForDomain(const char * domain, JS::MutableHandleValue _retval); 

/* Use this macro to declare functions that forward the behavior of this interface to another object. */
#define NS_FORWARD_NSIPUSHNOTIFICATIONSERVICE(_to) \
  NS_IMETHOD Register(const char * scope, JS::HandleValue originAttributes, JS::MutableHandleValue _retval) override { return _to Register(scope, originAttributes, _retval); } \
  NS_IMETHOD Unregister(const char * scope, JS::HandleValue originAttributes, JS::MutableHandleValue _retval) override { return _to Unregister(scope, originAttributes, _retval); } \
  NS_IMETHOD Registration(const char * scope, JS::HandleValue originAttributes, JS::MutableHandleValue _retval) override { return _to Registration(scope, originAttributes, _retval); } \
  NS_IMETHOD ClearAll(JS::MutableHandleValue _retval) override { return _to ClearAll(_retval); } \
  NS_IMETHOD ClearForDomain(const char * domain, JS::MutableHandleValue _retval) override { return _to ClearForDomain(domain, _retval); } 

/* Use this macro to declare functions that forward the behavior of this interface to another object in a safe way. */
#define NS_FORWARD_SAFE_NSIPUSHNOTIFICATIONSERVICE(_to) \
  NS_IMETHOD Register(const char * scope, JS::HandleValue originAttributes, JS::MutableHandleValue _retval) override { return !_to ? NS_ERROR_NULL_POINTER : _to->Register(scope, originAttributes, _retval); } \
  NS_IMETHOD Unregister(const char * scope, JS::HandleValue originAttributes, JS::MutableHandleValue _retval) override { return !_to ? NS_ERROR_NULL_POINTER : _to->Unregister(scope, originAttributes, _retval); } \
  NS_IMETHOD Registration(const char * scope, JS::HandleValue originAttributes, JS::MutableHandleValue _retval) override { return !_to ? NS_ERROR_NULL_POINTER : _to->Registration(scope, originAttributes, _retval); } \
  NS_IMETHOD ClearAll(JS::MutableHandleValue _retval) override { return !_to ? NS_ERROR_NULL_POINTER : _to->ClearAll(_retval); } \
  NS_IMETHOD ClearForDomain(const char * domain, JS::MutableHandleValue _retval) override { return !_to ? NS_ERROR_NULL_POINTER : _to->ClearForDomain(domain, _retval); } 

#if 0
/* Use the code below as a template for the implementation class for this interface. */

/* Header file */
class nsPushNotificationService : public nsIPushNotificationService
{
public:
  NS_DECL_ISUPPORTS
  NS_DECL_NSIPUSHNOTIFICATIONSERVICE

  nsPushNotificationService();

private:
  ~nsPushNotificationService();

protected:
  /* additional members */
};

/* Implementation file */
NS_IMPL_ISUPPORTS(nsPushNotificationService, nsIPushNotificationService)

nsPushNotificationService::nsPushNotificationService()
{
  /* member initializers and constructor code */
}

nsPushNotificationService::~nsPushNotificationService()
{
  /* destructor code */
}

/* jsval register (in string scope, in jsval originAttributes); */
NS_IMETHODIMP nsPushNotificationService::Register(const char * scope, JS::HandleValue originAttributes, JS::MutableHandleValue _retval)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* jsval unregister (in string scope, in jsval originAttributes); */
NS_IMETHODIMP nsPushNotificationService::Unregister(const char * scope, JS::HandleValue originAttributes, JS::MutableHandleValue _retval)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* jsval registration (in string scope, in jsval originAttributes); */
NS_IMETHODIMP nsPushNotificationService::Registration(const char * scope, JS::HandleValue originAttributes, JS::MutableHandleValue _retval)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* jsval clearAll (); */
NS_IMETHODIMP nsPushNotificationService::ClearAll(JS::MutableHandleValue _retval)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* jsval clearForDomain (in string domain); */
NS_IMETHODIMP nsPushNotificationService::ClearForDomain(const char * domain, JS::MutableHandleValue _retval)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* End of implementation class template. */
#endif


/* starting interface:    nsIPushQuotaManager */
#define NS_IPUSHQUOTAMANAGER_IID_STR "a2555e70-46f8-4b52-bf02-d978b979d143"

#define NS_IPUSHQUOTAMANAGER_IID \
  {0xa2555e70, 0x46f8, 0x4b52, \
    { 0xbf, 0x02, 0xd9, 0x78, 0xb9, 0x79, 0xd1, 0x43 }}

class NS_NO_VTABLE nsIPushQuotaManager : public nsISupports {
 public:

  NS_DECLARE_STATIC_IID_ACCESSOR(NS_IPUSHQUOTAMANAGER_IID)

  /* void notificationForOriginShown (in string origin); */
  NS_IMETHOD NotificationForOriginShown(const char * origin) = 0;

  /* void notificationForOriginClosed (in string origin); */
  NS_IMETHOD NotificationForOriginClosed(const char * origin) = 0;

};

  NS_DEFINE_STATIC_IID_ACCESSOR(nsIPushQuotaManager, NS_IPUSHQUOTAMANAGER_IID)

/* Use this macro when declaring classes that implement this interface. */
#define NS_DECL_NSIPUSHQUOTAMANAGER \
  NS_IMETHOD NotificationForOriginShown(const char * origin) override; \
  NS_IMETHOD NotificationForOriginClosed(const char * origin) override; 

/* Use this macro when declaring the members of this interface when the
   class doesn't implement the interface. This is useful for forwarding. */
#define NS_DECL_NON_VIRTUAL_NSIPUSHQUOTAMANAGER \
  NS_METHOD NotificationForOriginShown(const char * origin); \
  NS_METHOD NotificationForOriginClosed(const char * origin); 

/* Use this macro to declare functions that forward the behavior of this interface to another object. */
#define NS_FORWARD_NSIPUSHQUOTAMANAGER(_to) \
  NS_IMETHOD NotificationForOriginShown(const char * origin) override { return _to NotificationForOriginShown(origin); } \
  NS_IMETHOD NotificationForOriginClosed(const char * origin) override { return _to NotificationForOriginClosed(origin); } 

/* Use this macro to declare functions that forward the behavior of this interface to another object in a safe way. */
#define NS_FORWARD_SAFE_NSIPUSHQUOTAMANAGER(_to) \
  NS_IMETHOD NotificationForOriginShown(const char * origin) override { return !_to ? NS_ERROR_NULL_POINTER : _to->NotificationForOriginShown(origin); } \
  NS_IMETHOD NotificationForOriginClosed(const char * origin) override { return !_to ? NS_ERROR_NULL_POINTER : _to->NotificationForOriginClosed(origin); } 

#if 0
/* Use the code below as a template for the implementation class for this interface. */

/* Header file */
class nsPushQuotaManager : public nsIPushQuotaManager
{
public:
  NS_DECL_ISUPPORTS
  NS_DECL_NSIPUSHQUOTAMANAGER

  nsPushQuotaManager();

private:
  ~nsPushQuotaManager();

protected:
  /* additional members */
};

/* Implementation file */
NS_IMPL_ISUPPORTS(nsPushQuotaManager, nsIPushQuotaManager)

nsPushQuotaManager::nsPushQuotaManager()
{
  /* member initializers and constructor code */
}

nsPushQuotaManager::~nsPushQuotaManager()
{
  /* destructor code */
}

/* void notificationForOriginShown (in string origin); */
NS_IMETHODIMP nsPushQuotaManager::NotificationForOriginShown(const char * origin)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* void notificationForOriginClosed (in string origin); */
NS_IMETHODIMP nsPushQuotaManager::NotificationForOriginClosed(const char * origin)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* End of implementation class template. */
#endif


#endif /* __gen_nsIPushNotificationService_h__ */
