/* THIS FILE IS AUTOGENERATED BY gen-usecounters.py - DO NOT EDIT */

#ifndef USE_COUNTER_DOM_METHOD
#define USE_COUNTER_DOM_METHOD(interface_, name_) // nothing
#define DEFINED_USE_COUNTER_DOM_METHOD
#endif


#ifndef USE_COUNTER_DOM_ATTRIBUTE
#define USE_COUNTER_DOM_ATTRIBUTE(interface_, name_) // nothing
#define DEFINED_USE_COUNTER_DOM_ATTRIBUTE
#endif


#ifndef USE_COUNTER_CSS_PROPERTY
#define USE_COUNTER_CSS_PROPERTY(interface_, name_) // nothing
#define DEFINED_USE_COUNTER_CSS_PROPERTY
#endif

USE_COUNTER_DOM_METHOD(SVGSVGElement, getElementById)
USE_COUNTER_DOM_ATTRIBUTE(SVGSVGElement, currentScale)
USE_COUNTER_CSS_PROPERTY(Fill, Fill)
USE_COUNTER_CSS_PROPERTY(FillOpacity, FillOpacity)
USE_COUNTER_DOM_METHOD(PushManager, subscribe)
USE_COUNTER_DOM_METHOD(PushSubscription, unsubscribe)
USE_COUNTER_DOM_ATTRIBUTE(Window, sidebar)
USE_COUNTER_DOM_METHOD(External, addSearchEngine)

#ifdef DEFINED_USE_COUNTER_DOM_METHOD
#undef DEFINED_USE_COUNTER_DOM_METHOD
#undef USE_COUNTER_DOM_METHOD
#endif


#ifdef DEFINED_USE_COUNTER_DOM_ATTRIBUTE
#undef DEFINED_USE_COUNTER_DOM_ATTRIBUTE
#undef USE_COUNTER_DOM_ATTRIBUTE
#endif


#ifdef DEFINED_USE_COUNTER_CSS_PROPERTY
#undef DEFINED_USE_COUNTER_CSS_PROPERTY
#undef USE_COUNTER_CSS_PROPERTY
#endif

