/* -*- Mode: C++; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* vim: set ts=2 et sw=2 tw=80: */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsIDOMUIEvent.idl"

[builtinclass, uuid(5adfd99b-d3c7-4c6d-92d1-527264a71b50)]
interface nsIDOMCompositionEvent : nsIDOMUIEvent
{
  readonly attribute DOMString data;
  readonly attribute DOMString locale;

  void initCompositionEvent(in DOMString typeArg,
                            in boolean canBubbleArg,
                            in boolean cancelableArg,
                            in nsIDOMWindow viewArg,
                            in DOMString dataArg,
                            in DOMString localeArg);
};
