/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsISupports.idl"

/**
 * nsIForcePending interface exposes a function that enables overwriting of the normal 
 * behavior for the channel's IsPending(), forcing 'true' to be returned.
 */

[noscript, uuid(2ac3e1ca-049f-44c3-a519-f0681f51e9b1)]
interface nsIForcePendingChannel : nsISupports
{

/**
     * forcePending(true) overrides the normal behavior for the 
     * channel's IsPending(), forcing 'true' to be returned. A call to
     * forcePending(false) reverts IsPending() back to normal behavior.
     */
    void forcePending(in boolean aForcePending);
};
