/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsISupports.idl"

[scriptable, uuid(75d14f5d-293d-4872-8a26-e79268de592f)]
interface nsIXSLTProcessorPrivate : nsISupports
{
  /**
   * This needs to be called if the XSLTProcessor is instantiated
   * through the XPCOM registry (i.e. using do_createInstance) and the
   * stylesheet uses xsl:import/xsl:include or the document() xpath function.
   */
  void init(in nsISupports global);

  /**
   * Disables all loading of external documents, such as from
   * <xsl:import> and document()
   * Defaults to off and is *not* reset by calls to reset()
   */
  const unsigned long DISABLE_ALL_LOADS = 1;

  /**
   * Flags for this processor. Defaults to 0. See individual flags above
   * for documentation for effect of reset()
   */
  attribute unsigned long flags;
};
